@@ -14,6 +14,7 @@ from .utils import (
     check_dns,
     try_quit_xquartz,
     check_port_occupied,
+    check_for_updates,
 )
 from .pysectools import zero, Pinentry, PINENTRY_PATH
 from .config_manager import (
@@ -62,7 +63,7 @@ class CustomSSH(pxssh.pxssh):
         new_args = dict(
             sync_original_prompt=False,
             auto_prompt_reset=False,
-            original_prompt=f"[#$]|{self.duo_pattern}",
+            original_prompt=f"[$]|{self.duo_pattern}",
         )
         if kwargs:
             kwargs.update(new_args)
@@ -75,6 +76,8 @@ class CustomSSH(pxssh.pxssh):
                 logger.debug(f"RUN: ssh {username}@{host}")
             elif dns_err == 2:
                 raise SSHError(f"Unable to resolve server: {host}")
+            # This doesn't seem to override the ssh options
+            #  (like PubkeyAuthentication), so fine to keep as default
             self.force_password = True
             self.silence_logs()
             result = super(CustomSSH, self).login(
@@ -95,11 +98,11 @@ class CustomSSH(pxssh.pxssh):
                 self._buffer = self.buffer_type()
                 if code is not None:
                     self.sendline(code)
-                    i = self.expect(["[#$]", self.duo_pattern])
+                    i = self.expect(["$", self.duo_pattern])
                     logger.debug(f"Found prompt #{i}")
                     if i == 1:  # search for prompt one more time
                         self.sendline()
-                        i = self.expect(["[#$]", self.duo_pattern])
+                        i = self.expect(["$", self.duo_pattern])
                         logger.debug(f"Found prompt #{i}")
                     if i == 1:
                         self.close()
@@ -345,6 +348,9 @@ class JupyterO2(object):
         self.password_request_pattern = re.compile(
             password_request_pattern.encode("utf-8")
         )
+        self.use_pubkey = config.getboolean(
+            "Remote Environment Settings", "USE_PUBLIC_KEY_AUTHENTICATION"
+        )
 
         if len(self.codes_2fa) == 1:
             print_code = self.codes_2fa[0]
@@ -435,7 +441,7 @@ class JupyterO2(object):
         login_ssh_options = {
             "ForwardX11": "yes",
             "LocalForward": f"{self.jp_port} 127.0.0.1:{self.jp_port}",
-            "PubkeyAuthentication": "no",
+            "PubkeyAuthentication": "yes" if self.use_pubkey else "no",
         }
         if forwardx11trusted:
             login_ssh_options["ForwardX11Trusted"] = "yes"
@@ -445,7 +451,9 @@ class JupyterO2(object):
         )
 
         self._second_ssh = CustomSSH(
-            timeout=10, ignore_sighup=False, options={"PubkeyAuthentication": "no"}
+            timeout=60,
+            ignore_sighup=False,
+            options={"PubkeyAuthentication": "yes" if self.use_pubkey else "no"},
         )
 
         # perform close() on exit or term() on interrupt
@@ -458,7 +466,10 @@ class JupyterO2(object):
         """
         Run the standard JupyterO2 sequence
         """
-        self.ask_for_pin()
+        if (
+            not self.use_pubkey
+        ):  # If using PubKey authentication, we don't need to use a password
+            self.ask_for_pin()
         if self.connect() or self.keep_alive:
             self.logger.debug("Starting pexpect interactive mode.")
             self.interact()
@@ -488,7 +499,13 @@ class JupyterO2(object):
         # start login ssh
         self.logger.info(f"Connecting to {self.user}@{self.host}")
         code = self.codes_2fa[0] if self.codes_2fa else None
-        if not self._login_ssh.login(self.host, self.user, self.__pass, code):
+        if not self._login_ssh.login(
+            self.host,
+            self.user,
+            self.__pass,
+            code,
+            login_timeout=self._login_ssh.timeout,
+        ):
             return False
         self.logger.debug("Connected.")
 
@@ -509,7 +526,13 @@ class JupyterO2(object):
             # log in to the second ssh
             self.logger.info("\nStarting a second connection to the login node.")
             code = self.codes_2fa[:2][-1] if self.codes_2fa else None
-            if not self._second_ssh.login(jp_login_host, self.user, self.__pass, code):
+            if not self._second_ssh.login(
+                jp_login_host,
+                self.user,
+                self.__pass,
+                code,
+                login_timeout=self._second_ssh.timeout,
+            ):
                 return False
             self.logger.debug("Connected.")
 
@@ -591,6 +614,7 @@ class JupyterO2(object):
                 [b"srun:", b"authenticity", b"unavailable"],
                 reactions={
                     b"authenticity": self.close_on_known_hosts_error,
+                    b"No DISPLAY variable set": self.log_x11_error,
                     b"srun: error:": self.close_on_srun_error,
                     b"in use or unavailable": self.close_on_port_unavailable,
                 },
@@ -633,27 +657,41 @@ class JupyterO2(object):
         Print a known_hosts error message and close.
         """
         self.logger.critical(
-            "\nCould not connect to interactive session.\n"
+            "Could not connect to interactive session.\n"
             "For some reason, the requested node is not recognized "
             "in ssh_known_hosts.\n"
             "If on O2, check with HMS RC."
         )
         self.term()
 
+    def log_x11_error(self):
+        """
+        Print an error message for an X11 error
+        """
+        if sys.platform == "darwin":
+            self.logger.critical(
+                "X11 error. "
+                "You may need to reinstall XQuartz. "
+                "Download from https://www.xquartz.org or use brew reinstall xquartz"
+            )
+        else:
+            self.logger.critical(
+                "X11 error. "
+                "You may need to reinstall X11 or export the DISPLAY variable."
+            )
+
     def close_on_srun_error(self):
         """
         Print a known_hosts error message and close.
         """
-        self.logger.critical(
-            "\nCould not start interactive session due to SLURM error."
-        )
+        self.logger.critical("Could not start interactive session due to SLURM error.")
         self.term()
 
     def close_on_port_unavailable(self):
         """
         Print a port unavailable error message and close.
         """
-        self.logger.critical("\nThe selected port appears to be unavailable.")
+        self.logger.critical("The selected port appears to be unavailable.")
         self.term()
 
     def ssh_into_interactive_node(self, s, interactive_host, sendpass=False):
@@ -796,6 +834,10 @@ def main():
 
     logger = logging.getLogger(__name__)
 
+    new_version = check_for_updates()
+    if new_version:
+        logger.info(f"A new version of jupyter-o2 is available ({new_version})")
+
     if not cfg_locations:
         logger.warning("Config file could not be read. Using internal defaults.")
     else:
